
using System;
using System.Collections.Generic;
using System.Text;

using ServiceProvider;

namespace ServiceManager
{
	/// <summary>
	/// This class is intended for recurrent daily activation at the specified time of day.
	/// </summary>
	public class DailyActivationController : IDisposable
	{
		protected const int MinHours = 0;
        protected const int MaxHours = 23;
        protected const int MinMinutes = 0;
        protected const int MaxMinutes = 59;
        protected const int MaxIntervalMinutes = 60;

        protected IServiceEventListener m_serviceListener;
        protected System.Timers.Timer m_timer;
        protected TimeSpan m_activationTime;

        protected DateTime m_lastVerified;
        protected DateTime m_lastActivated;

        protected bool m_disposed = false;

		public event EventHandler Activate;

		/// <summary>
		/// Public constructor of the class.
		/// </summary>
		/// <param name="serviceListener">The object that implements IServiceEventListener interface.</param>
		public DailyActivationController(IServiceEventListener serviceListener)
		{
			m_serviceListener = serviceListener;

			m_timer = new System.Timers.Timer();
			m_timer.Elapsed += new System.Timers.ElapsedEventHandler(TimerElapsed);
		}

		~DailyActivationController()
		{
			Dispose(false);
		}

		/// <summary>
		/// Starts waiting the activation time.
		/// </summary>
		public void Start(int activationTimeHours, int activationTimeMinutes, int minutesInterval)
		{
			m_activationTime = GetTime(activationTimeHours, activationTimeMinutes);
			m_timer.Interval = GetInterval(minutesInterval);

			m_lastActivated = DateTime.MinValue;
			m_lastVerified = DateTime.MinValue;

			CheckForActivate();
			m_timer.Start();
		}

		/// <summary>
		/// Stops waiting the activation time.
		/// </summary>
		public void Stop()
		{
			m_timer.Stop();
		}

		/// <summary>
		/// Performs disposing unmanaged resources.
		/// </summary>
		public void Dispose()
		{
			Dispose(true);
			GC.SuppressFinalize(this);
		}

		protected virtual void Dispose(bool disposing)
		{
			if (!m_disposed && (null != m_timer))
			{
				m_timer.Dispose();
			}
			m_disposed = true;
		}

        protected static TimeSpan GetTime(int hours, int minutes)
		{
			if (hours > MaxHours || hours < MinHours)
			{
				throw new Exception(ExceptionMessages.IncorrectHoursValueMessage(Convert.ToString(MaxHours), Convert.ToString(MinHours)));
			}

			if (minutes > MaxMinutes || minutes < MinMinutes)
			{
				throw new Exception(ExceptionMessages.IncorrectMinutesValueMessage(Convert.ToString(MaxMinutes), Convert.ToString(MinMinutes)));
			}

			return new TimeSpan(hours, minutes, 0);
		}

        protected static double GetInterval(int minutesInterval)
		{
			if (minutesInterval <= 0 || minutesInterval > MaxIntervalMinutes)
			{
				throw new Exception(ExceptionMessages.IncorrectIntervalValueMessage(Convert.ToString(MaxIntervalMinutes)));
			}

			return minutesInterval * 60 * 1000;
		}

        protected bool VerifyActivationTime(DateTime currentDateTime)
		{
			bool result = IsNotActivatedDuringCurrentDay(currentDateTime) && IsTimeToActivate(currentDateTime);

			m_lastVerified = currentDateTime;

			return result;
		}

        protected bool IsNotActivatedDuringCurrentDay(DateTime currentDateTime)
		{
			return m_lastActivated.Date < currentDateTime.Date;
		}

        protected bool IsTimeToActivate(DateTime currentDateTime)
		{
			return IsActivationTime(currentDateTime) ||
				IsTimeAfterActivationTime(currentDateTime) ||
				IsMissedTheDayBefore(currentDateTime);
		}

        protected bool IsActivationTime(DateTime currentDateTime)
		{
			return currentDateTime.TimeOfDay == m_activationTime;
		}

        protected bool IsTimeAfterActivationTime(DateTime currentDateTime)
		{
			return (currentDateTime.TimeOfDay > m_activationTime) && (m_activationTime > m_lastVerified.TimeOfDay) && (DateTime.MinValue != m_lastVerified);
		}

        protected bool IsMissedTheDayBefore(DateTime currentDateTime)
		{
			return (DateTime.MinValue != m_lastVerified) && (currentDateTime.Date > m_lastVerified.Date) && (m_activationTime >= m_lastVerified.TimeOfDay);
		}

        protected void CheckForActivate()
		{
			if (VerifyActivationTime(DateTime.Now))
			{
				if (null != Activate)
				{
					Activate(this, null);
				}
				m_lastActivated = m_lastVerified;
			}
		}

        protected void TimerElapsed(object sender, System.Timers.ElapsedEventArgs e)
		{
			try
			{
				CheckForActivate();
			}
			catch (Exception ex)
			{
				if (null != m_serviceListener)
				{
					m_serviceListener.LogMessage(ex.Message);
				}
			}
		}
	}
}