
using System;
using System.Diagnostics;
using System.Diagnostics.CodeAnalysis;
using System.Runtime.Remoting;
using System.Reflection;
using System.IO;

using ServiceProvider;
using System.Collections.Generic;

namespace ServiceManager
{

    public delegate void LogDelegate(string s);
    public delegate void LogExcDelegate(Exception e);

    /// <summary>
    /// Singleton which loads services and contains them.
    /// </summary>
    public class ServiceLoader : IDisposable, IServiceEventListener
    {
        private static List<ServiceInfo> m_serviceList = new List<ServiceInfo>();

        private static ServiceLoader _instance = null;

        #region Delegate Definitions

        private static LogDelegate m_logDelegate;
        private static LogExcDelegate m_logExcDelegate;

        #endregion

        public static ServiceLoader Instance
        {
            get
            {
                if (_instance == null)
                {
                    throw new NullReferenceException("Instance has not been initialized.");
                }
                return _instance;
            }
        }

        public static bool InstanceInitialized
        {
            get
            {
                return _instance != null;
            }
        }

        public static ServiceLoader CreateInstance(LogDelegate logDelegate)
        {
            if (_instance != null)
            {
                throw new InvalidOperationException("Instance has been already created.");
            }

            m_logDelegate = logDelegate;
            _instance = new ServiceLoader();

            return Instance;
        }

        public static ServiceLoader CreateInstance(LogDelegate logDelegate, LogExcDelegate exceptionDelegate)
        {
            if (_instance != null)
            {
                throw new InvalidOperationException("Instance has been already created.");
            }

            m_logDelegate = logDelegate;
            m_logExcDelegate = exceptionDelegate;
            _instance = new ServiceLoader();

            return Instance;
        }

        #region Construction/Initialization/Destruction

        public ServiceLoader()
        {
            Initialize();
        }

        // Added this constructor for testing purposes
        public ServiceLoader(LogDelegate logDelegate, LogExcDelegate exceptionDelegate)
        {
            m_logDelegate = logDelegate;
            m_logExcDelegate = exceptionDelegate;
        }

        public void InitializeService()
        {
            Initialize();
        }

        #region IDisposable Members

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        ~ServiceLoader()
        {
            Dispose(false);
        }

        protected virtual void Dispose(bool disposing)
        {
            if (disposing)
            {
                ShutdownServices();
            }
        }

        #endregion

        #endregion

        protected void Initialize()
        {
            try
            {
                LogEntry("Service manager started successfully.");
                CreateRemoteServer();
                LoadServices();
            }
            catch (Exception e)
            {
                LogEntry("Error initializing the service manager... (" + e.Message + ").");
                throw;
            }
        }

        public void CreateRemoteServer()
        {
            string remotingConfigPath = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location) + "\\RemotingServer.config";
            if (RemotingConfiguration.ApplicationName != "ServiceManager")
            {
                RemotingConfiguration.Configure(remotingConfigPath, false);
            }
        }

        #region Logging
        public void LogMessage(string message)
        {
            LogEntry(message);
        }

        public void LogException(Exception ex)
        {
            LogExceptionEntry(ex);
        }

        [SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        internal static void LogEntry(string entry)
        {
            try
            {
                Debug.Assert(null != entry);

                if (entry.Length > 0)
                {
                    if (null != m_logDelegate)
                    {
                        m_logDelegate(entry);
                    }
                    else
                    {
                        throw new InvalidOperationException("Log delegate has not been initialized.");
                    }
                }
            }
            catch (Exception e)
            {
                //Logging entry should NEVER throw exception
                Debug.Assert(false, e.Message);
            }
        }

        [SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        internal void LogExceptionEntry(Exception ex)
        {
            try
            {
                Debug.Assert(null != ex);

                if (null != m_logExcDelegate)
                {
                    m_logExcDelegate(ex);
                }
                else
                {
                    throw new InvalidOperationException("Log exception delegate has not been initialized.");
                }
            }
            catch (Exception e)
            {
                //Logging entry should NEVER throw exception
                Debug.Assert(false, e.Message);
            }
        }

        #endregion Logging

        protected void LoadServices()
        {
            try
            {
                ServiceConfigManager configManager = ServiceConfigManager.Create();
                for (int i = 0; i < configManager.Count; i++)
                {
                    string assemblyName = configManager[i];

                    FetchAndStartServicesInAssembly(assemblyName);
                }
            }
            catch (Exception e)
            {
                LogEntry("Error loading services (" + e.Message + ").");
                throw;
            }
        }

        [SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void FetchAndStartServicesInAssembly(string assemblyName)
        {
            try
            {
                ServiceAssemblyFactory builder = new ServiceAssemblyFactory();
                ServiceAssembly assembly = builder.Build(assemblyName, this);

                foreach (Service service in assembly)
                {
                    ServiceInfo si = new ServiceInfo(service.Name, false, service);
                    m_serviceList.Add(si);
                    if (service.AutoStart)
                    {
                        ChangeServiceStatus(si, true);
                    }
                }
            }
            catch (Exception e)
            {
                LogExceptionEntry(e);
                System.Diagnostics.Debug.WriteLine(e.Message);
                //looks like we DO NOT need throw expceptin here...
                //we need only skip particular parsing assembly
            }
        }

        public void ShutdownServices()
        {
            for (int i = 0; i < m_serviceList.Count; i++)
            {
                ChangeServiceStatus(m_serviceList[i], false);
            }
        }


        public void ServiceStarted(Service service)
        {
            LogEntry(service.Name + " started successfully.");
        }

        public void ServiceFinished(ServiceResult result)
        {
            LogEntry(result.Name + " stopped successfully.");
        }

        public void ChangeServiceStatus(IServiceInfo si, bool start)
        {
            ServiceResult result = null;

            if (null != si.Service)
            {
                switch (start)
                {
                    case true:
                        result = new ServiceResult(si.ServiceName);
                        if (null != si.Service.Run(this))
                        {
                            si.Started = true;
                        }
                        break;
                    case false:
                        if (null != si.Service)
                        {
                            result = new ServiceResult(si.ServiceName);

                            if (si.Service.Stop(result, this))
                            {
                                si.Started = false;
                            }
                        }
                        break;
                }
            }
        }

        public ServiceInfo this[int index]
        {
            get
            {
                return m_serviceList[index];
            }
            set
            {
                m_serviceList[index] = value;
            }
        }

        public int Count
        {
            get
            {
                return m_serviceList.Count;
            }
        }

        public IList<ServiceInfo> ServicesList
        {
            get
            {
                return m_serviceList;
            }
        }

        public LogDelegate LogCallback
        {
            get
            {
                return m_logDelegate;
            }
            set
            {
                m_logDelegate = value;
            }
        }

        public LogExcDelegate LogExceptionCallback
        {
            get
            {
                return m_logExcDelegate;
            }
            set
            {
                m_logExcDelegate = value;
            }
        }
    }
}
