
using System;
using System.Reflection;
using System.Collections;
using ServiceManager.Exceptions;

namespace ServiceProvider
{
    /// <summary>
    /// Summary description for Service.
    /// </summary>
    public class Service : AbstractService, IServiceImpl
    {
        private object m_serviceObject;
        private bool m_autoStart = false;

        protected Service(string name)
            : base(name)
        {
        }

        protected Service(string path, string name)
            : base(path, name)
        {
        }

        public Service(object service)
            : this(service.GetType().FullName)
        {
            m_serviceObject = service;

        }

        public override ServiceResult Run(IServiceEventListener listener)
        {
            ServiceResult result = null;

            ServiceResult serviceResult = new ServiceResult(this.Name);

            try
            {
                IService serviceInterface = (IService)m_serviceObject;
                serviceInterface.ServiceEventListener = listener;
            }
            catch (Exception)
            {
                // Ignore, because the service does not support the event listener.
            }

            if (!this.Started)
            {
                if (null != listener)
                    listener.ServiceStarted(this);

                serviceResult.StartTime = DateTime.Now;

                if (Run(serviceResult))
                {
                    result = serviceResult;
                    this.Started = true;
                }
            }
            else
                result = serviceResult;

            return result;
        }

        public bool Stop(ServiceResult serviceResult, IServiceEventListener listener)
        {
            bool result = Stop(serviceResult);

            if (this.Started)
            {
                if (null != listener)
                {
                    listener.ServiceFinished(serviceResult);
                    this.Started = false;
                }
            }

            return result;
        }

        public bool Run(ServiceResult serviceResult)
        {
            bool result = false;

            try
            {
                result = InvokeStart();
            }
            catch (Exception exp)
            {
                throw new UnableToStartServiceException("While trying to find the start method on the provided service one was not found.");
            }

            return result;
        }

        public bool Stop(ServiceResult serviceResult)
        {
            bool result = false;

            try
            {
                result = InvokeStop();
            }
            catch (Exception exp)
            {
                throw new UnableToStopServiceException("While trying to find the stop method on the provided service one was not found.");
            }
            finally
            {
                if (null != serviceResult)
                    serviceResult.StopTime = DateTime.Now;
            }

            return result;
        }

        private bool InvokeStart()
        {
            bool result = false;

            MethodInfo method = FindStartMethod(m_serviceObject);
            if (method != null)
            {
                try
                {
                    method.Invoke(m_serviceObject, null);
                    result = true;
                }
                catch (TargetInvocationException e)
                {
                    throw new UnableToStartServiceException( "While trying to call the start method on the provided service there was an error", e );
                }
            }

            return result;
        }

        private bool InvokeStop()
        {
            bool result = false;

            MethodInfo method = FindStopMethod(m_serviceObject);
            if (method != null)
            {
                try
                {
                    method.Invoke(m_serviceObject, null);
                    result = true;
                }
                catch (TargetInvocationException e)
                {
                    throw new UnableToStopServiceException("While trying to call the stop method on the provided service there was an error", e);
                }
            }

            return result;
        }

        private MethodInfo FindStartMethod(object service)
        {
            return FindMethodByAttribute(service, typeof(ServiceProvider.ServiceStartAttribute));
        }

        private MethodInfo FindStopMethod(object service)
        {
            return FindMethodByAttribute(service, typeof(ServiceProvider.ServiceStopAttribute));
        }

        private MethodInfo FindMethodByAttribute(object service, Type type)
        {
            foreach (MethodInfo method in service.GetType().GetMethods(BindingFlags.Public | BindingFlags.Instance | BindingFlags.NonPublic))
            {
                if (method.IsDefined(type, true))
                {
                    return method;
                }
            }

            return null;
        }
        public bool AutoStart
        {
            get { return m_autoStart; }
            set { m_autoStart = value; }
        }

        public MarshalByRefObject GetServiceConfigurator()
        {
            if (m_serviceObject is IServiceConfigurable)
            {
                return (m_serviceObject as IServiceConfigurable).GetConfigurator();
            }
            throw new ConfiguratorNotSupportedException(base.FullName);
        }

    }

}
