
using System;
using System.IO;
using System.Reflection;
using System.Collections;

namespace ServiceProvider
{
	/// <summary>
	/// SeviceAssemblyBuilder.
	/// </summary>
	public class ServiceAssemblyFactory
	{
		public ServiceAssembly Build(string assemblyName, IServiceEventListener listener)
		{
			bool isServicePresent = false;
			ServiceAssembly result = null;

			Assembly assembly	= Load(assemblyName);
			if(null != assembly)
			{
				result = new ServiceAssembly(assemblyName);

				Type[] serviceTypes = assembly.GetExportedTypes();
				foreach(Type serviceType in serviceTypes)
				{
					if(IsService(serviceType))
					{
						isServicePresent = true;
						PrepareService(listener, result, serviceType);
					}
				}

				if (!isServicePresent)
					System.Diagnostics.Debug.WriteLine(assemblyName + " does not have any services");
			}

			return result;
		}

		private void PrepareService(IServiceEventListener listener, ServiceAssembly result, Type serviceType)
		{
						try
						{
				IService service = (IService)LoadService(serviceType);

				string reason;
				if (IsNeededToBeStarted(service, serviceType, out reason))
				{
							result.Add(service);
						}
				else
				{
					LogNotStartedService(serviceType, listener, reason);
					if ((null != service) && (service is IDisposable))
						{
						((IDisposable)service).Dispose();
						}
					}
				}
			catch (Exception exception)
			{
				System.Diagnostics.Debug.WriteLine(serviceType, exception.Message);
			}
		}

		private object LoadService(Type serviceType)
		{
			ConstructorInfo ctor = serviceType.GetConstructor(Type.EmptyTypes);
			if(ctor == null) 
				System.Diagnostics.Debug.WriteLine(serviceType.FullName + " does not have a valid constructor");

			object service = ctor.Invoke(Type.EmptyTypes);
			if(service == null) 
				System.Diagnostics.Debug.WriteLine(ctor.Name + " cannot be invoked");

			if(HasMultipleStartMethods(service))
			{
				System.Diagnostics.Debug.WriteLine(ctor.Name + " has multiple start methods");
			}
			if(HasMultipleStopMethods(service))
			{
				System.Diagnostics.Debug.WriteLine(ctor.Name + " has multiple stop methods");
			}

			return service;
		}

		private int CountMethodWithGivenAttribute(object service, Type type)
		{
			int count = 0;
			foreach(MethodInfo method in service.GetType().GetMethods(BindingFlags.Public|BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.DeclaredOnly))
			{
				if(method.IsDefined(type,false)) 
					count++;
			}
			return count;

		}

		private bool HasMultipleStartMethods(object service)
		{
			return CountMethodWithGivenAttribute(service,typeof(ServiceProvider.ServiceStartAttribute)) > 1;
		}

		private bool HasMultipleStopMethods(object service)
		{
			return CountMethodWithGivenAttribute(service,typeof(ServiceProvider.ServiceStopAttribute)) > 1;
		}

		private bool IsService(Type type)
		{
			if(type.IsAbstract) 
				return false;

			return type.IsDefined(typeof(ServiceProvider.ServiceAttribute), true) &&
				(typeof(IService) == type.GetInterface(typeof(IService).ToString(), true));
		}

		public Assembly Load(string assemblyName)
		{
			Assembly assembly = AppDomain.CurrentDomain.Load(TrimPathAndExtension(assemblyName));
			return assembly;
		}

		public string TrimPathAndExtension(string assemblyName) 
		{
			FileInfo info = new FileInfo(assemblyName);
			string name = info.Name;
			string extension = info.Extension;
			if (extension != String.Empty)
				name = name.Substring(0, name.IndexOf(extension));

			return name;
		}

		private static bool IsNeededToBeStarted(IService service, Type serviceType, out string reason)
		{
			bool isNeededToBeStarted = false;
			reason = null;

			if (null != service)
			{
				isNeededToBeStarted = service.IsNeededToBeStarted(out reason);				
			}
			return isNeededToBeStarted;
		}

		private static void LogNotStartedService(Type serviceType, IServiceEventListener listener, string reason)
		{
			const string ServiceCannotBeStartedFormat = "{0} can not be started: {1}";
			if (null != listener)
			{
				if (null == reason)
				{
					reason = string.Empty;
				}
				listener.LogMessage(string.Format(ServiceCannotBeStartedFormat, serviceType.FullName, reason));
			}
		}
	}
}
